"""
pyweblib.helper - Misc. stuff useful in CGI-BINs
(c) by Michael Stroeder <michael@stroeder.com>
"""

__version__ = '0.2.0'

import sys, os, string, re, UserDict

known_browsers = {
                   'MSIE':'Microsoft Internet Explorer',
		   'Mozilla':'Netscape Navigator',
		   'Lynx':'Lynx',
		   'Opera':'Opera',
		   'StarOffice':'StarOffice',
		   'NCSA_Mosaic':'NCSA Mosaic',
		   'NetPositive':'Net Positive'
                 }
known_browsers_rev = {}
for b in known_browsers.keys():
  known_browsers_rev[known_browsers[b]]=b

compatible_browsers = known_browsers.keys()
compatible_browsers.remove('Mozilla')

compatible_browsers_re = re.compile('(%s)[/ ]+([0-9.]*)' % string.join(compatible_browsers,'|'))
mozilla_re             = re.compile('(Mozilla)[/ ]+([0-9.]*)')


def BrowserType(http_user_agent):
  """
  Parse the HTTP_USER_AGENT environment variable and return the
  tuple (Browser,Version).

  Not sure if this succeeds in every situation since most
  browsers have very obscure HTTP_USER_AGENT entries for compability reasons.
  The following browsers are known by name:
  Netscape	Netscape Navigator, Netscape Communicator)
  MSIE		MS Internet Explorer
  Opera		Opera browser from http://www.operasoftware.com/
  StarOffice	built-in browser of Star Office
  Lynx		the text-based browser Lynx
  NetPositive	Net Positive (BeOS)
  """

  if not http_user_agent:
    return ('','')
  else:
    browserrm = compatible_browsers_re.search(http_user_agent)
    if browserrm:
      return browserrm.groups()
    else:
      browserrm = mozilla_re.search(http_user_agent)
      if browserrm:
        return browserrm.groups()
      else:
        return ('','')


class AcceptCharsetDict(UserDict.UserDict):
  """
  This dictionary class is used to determine the character
  set capabilities of a browser by parsing HTTP header
  Accept-Charset.
  
  See RFC2616 section "14.2 Accept-Charset".
  """

  def __init__(self,env=os.environ):
    """
    Parse the HTTP_ACCEPT_CHARSET env var into an internal
    dictionary.

    env
        dictionary holding environment vars
    """
    UserDict.UserDict.__init__(self)
    self.preferred_charset = []
    try:
      http_accept_charset = [
        s
        for s in env['HTTP_ACCEPT_CHARSET'].strip().split(',')
        if len(s)
      ]
    except KeyError:
      self.data = {'*':1.0}
    else:
      if not http_accept_charset:
        self.data = {'*':1.0}
      else:
        self.data = {}
        for i in http_accept_charset:
          try:
            c,w=i.split(';')
          except ValueError:
            c,w = i,''
          # Normalize charset name
          c=c.strip().lower()
          try:
            q,qvalue_str=w.split('=',1)
            qvalue = float(qvalue_str)
          except ValueError:
            qvalue = 1.0
          # Add to capability dictionary
          if c:
            self.data[c] = qvalue
        # Special treating of ISO-8859-1 charset to be compliant to RFC2616
        if not self.data.has_key('iso-8859-1'):
          self.data['iso-8859-1'] = 1.0

  def __getitem__(self,charset):
    """
    charset
        String representing the name of the charset for which to return
        the floating point capability value (see RFC2616, section 3.4).
    """
    return self.data.get(
      charset.lower(),
      self.data.get('*',0)
    )

  def preferredCharset(self,defaultCharset=None):
    """
    Return the charset name with highest capabilty weigth
    
    defaultCharset
        If not None and '*' would be the result this charset name
        is returned instead of '*'.
    """
    l = self.items()
    l.sort(lambda x,y:cmp(y[1],x[1]))
    while l and l[0][0]!='*':
      try:
        unicode('').encode(l[0][0])
      except LookupError:
        l.pop(0)
      else:
        break
    if l:
      if defaultCharset and l[0][0]=='*':
        return defaultCharset
      else:
        return l[0][0]
    else:
      return 'iso-8859-1'
